﻿using Kivii.Finances.Entities;
using Kivii.Office.OpenXml.Excel;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Addons
{
    public class ReadExcelResponse<T>:RestfulQueryResponse<T>
    {
        public bool success { get; set; }
    }

    [Api(Description = "读取Excel收款流水")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentReadReceiptExcel : RestfulExecution<Payment>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            (Request.Files.Count() < 1 || Request.Files == null).ThrowIfTrue("未找到excel文件!");
            var httpFile = Request.Files[0];
            ExcelPackage package = new ExcelPackage(Request.Files[0].InputStream);
            (package.Workbook.Worksheets.Count == 0).ThrowIfTrue("未找到Sheet!");

            var rtns = new ReadExcelResponse<Payment>();
            rtns.Results = new List<Payment>();
            try
            {
                var sheet = package.Workbook.Worksheets[1];
                var end = sheet.Dimension.End.Row;
                for (int i = 2; i <= end; i++)
                {
                    #region 格式化Excel中每行数据,必填项若不符合验证条件就抛出异常
                    var eSerialNumber = sheet.Cells[i, 2].Value;
                    if(eSerialNumber==null)
                    {
                        throw new Exception($"Excel第{i}行第2列不可为空,请传入不重复的流水号!");
                    }
                    var serialNumber = eSerialNumber.ToString();
                    var ePayerName = sheet.Cells[i, 3].Value;
                    if (ePayerName == null)
                    {
                        throw new Exception($"Excel第{i}行第3列不可为空,请传入付款单位!");
                    }
                    var payerName = ePayerName.ToString();
                    var eType = sheet.Cells[i, 4].Value;
                    if (eType == null)
                    {
                        throw new Exception($"Excel第{i}行第4列不可为空,请选择指定类型!");
                    }
                    if(!Enum.TryParse(eType.ToString(), true, out PaymentType type))
                    {
                        throw new Exception($"Excel第{i}行第4列类型不正确,请选择指定类型!");
                    }
                    if (type != PaymentType.Bank && type != PaymentType.Cash && type != PaymentType.Pos && type != PaymentType.AliPay && type != PaymentType.WeChat)
                    {
                        throw new Exception($"Excel第{i}行第4列类型不正确,请选择指定类型!");
                    }
                    var eAmount = sheet.Cells[i, 5].Value;
                    if (eAmount == null)
                    {
                        throw new Exception($"Excel第{i}行第5列不可为空,请传入发生金额!");
                    }
                    if (!Decimal.TryParse(eAmount.ToString(),out var amount))
                    {
                        throw new Exception($"Excel第{i}行第5列金额格式不正确!");
                    }
                    if(amount <= 0)
                    {
                        throw new Exception($"Excel第{i}行第5列金额不能小于等于零!");
                    }
                    var eCurrency = sheet.Cells[i, 6].Value;
                    if (eCurrency == null)
                    {
                        throw new Exception($"Excel第{i}行第6列不可为空,请传入货币单位!");
                    }
                    if (!Enum.TryParse(eCurrency.ToString(),true, out CurrencyUnit currency))
                    {
                        throw new Exception($"Excel第{i}行第6列货币单位格式不正确!");
                    }
                    if (currency != CurrencyUnit.CNY)
                    {
                        throw new Exception($"Excel第{i}行第6列货币单位格式不正确,请选择指定货币单位!");
                    }
                    var eOperateTime = sheet.Cells[i, 7].Value;
                    if (eOperateTime == null)
                    {
                        throw new Exception($"Excel第{i}行第7列不可为空,请传入发生日期!");
                    }
                    if (!DateTime.TryParse(eOperateTime.ToString(), out DateTime operateTime))
                    {
                        throw new Exception($"Excel第{i}行第7列发生日期格式不正确!");
                    }
                    var eBank = sheet.Cells[i, 8].Value;
                    var bank = eBank == null ? string.Empty : eBank.ToString();
                    var eBankNumber = sheet.Cells[i, 9].Value;
                    var bankNumber = eBankNumber == null ? string.Empty : eBankNumber.ToString();
                    var eSummary = sheet.Cells[i, 10].Value;
                    var summary = eSummary == null ? string.Empty : eSummary.ToString();
                    var eRemark = sheet.Cells[i, 11].Value;
                    var remark = eRemark == null ? string.Empty : eRemark.ToString();
                    #endregion

                    var payment = new Payment();
                    payment.Metadata = new Dictionary<string, string>();
                    payment.SerialNumber = serialNumber;
                    payment.Type = type;
                    payment.Currency = currency;
                    payment.OperateTime = operateTime;
                    payment.Amount = amount;
                    payment.PayerName = payerName;
                    payment.PayerAccountName = bank;
                    payment.PayerAccountSerialNumber = bankNumber;
                    payment.Summary = summary;
                    payment.Remark = remark;
                    rtns.Results.Add(payment);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            rtns.Total = rtns.Results.Count;
            rtns.success = true;
            return rtns;
        }
    }
}
